#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

import unittest
from datetime import datetime
from unittest.mock import Mock, PropertyMock

from airbyte_cdk.sources.file_based.availability_strategy.default_file_based_availability_strategy import (
    DefaultFileBasedAvailabilityStrategy,
)
from airbyte_cdk.sources.file_based.config.file_based_stream_config import FileBasedStreamConfig
from airbyte_cdk.sources.file_based.config.jsonl_format import JsonlFormat
from airbyte_cdk.sources.file_based.file_based_stream_reader import AbstractFileBasedStreamReader
from airbyte_cdk.sources.file_based.file_types.file_type_parser import FileTypeParser
from airbyte_cdk.sources.file_based.remote_file import RemoteFile
from airbyte_cdk.sources.file_based.stream import AbstractFileBasedStream

_FILE_WITH_UNKNOWN_EXTENSION = RemoteFile(uri="a.unknown_extension", last_modified=datetime.now(), file_type="csv")
_ANY_CONFIG = FileBasedStreamConfig(
    name="config.name",
    file_type="parquet",
    format=JsonlFormat(),
)
_ANY_SCHEMA = {"key": "value"}


class DefaultFileBasedAvailabilityStrategyTest(unittest.TestCase):
    def setUp(self) -> None:
        self._stream_reader = Mock(spec=AbstractFileBasedStreamReader)
        self._strategy = DefaultFileBasedAvailabilityStrategy(self._stream_reader)

        self._parser = Mock(spec=FileTypeParser)
        self._stream = Mock(spec=AbstractFileBasedStream)
        self._stream.get_parser.return_value = self._parser
        self._stream.catalog_schema = _ANY_SCHEMA
        self._stream.config = _ANY_CONFIG
        self._stream.validation_policy = PropertyMock(validate_schema_before_sync=False)
        self._stream.stream_reader = self._stream_reader

    def test_given_file_extension_does_not_match_when_check_availability_and_parsability_then_stream_is_still_available(self) -> None:
        """
        Before, we had a validation on the file extension but it turns out that in production, users sometimes have mismatch there. The
        example we've seen was for JSONL parser but the file extension was just `.json`. Note that there we more than one record extracted
        from this stream so it's not just that the file is one JSON object
        """
        self._stream.get_files.return_value = [_FILE_WITH_UNKNOWN_EXTENSION]
        self._parser.parse_records.return_value = [{"a record": 1}]

        is_available, reason = self._strategy.check_availability_and_parsability(self._stream, Mock(), Mock())

        assert is_available

    def test_not_available_given_no_files(self) -> None:
        """
        If no files are returned, then the stream is not available.
        """
        self._stream.get_files.return_value = []

        is_available, reason = self._strategy.check_availability_and_parsability(self._stream, Mock(), Mock())

        assert not is_available
        assert "No files were identified in the stream" in reason

    def test_parse_records_is_not_called_with_parser_max_n_files_for_parsability_set(self) -> None:
        """
        If the stream parser sets parser_max_n_files_for_parsability to 0, then we should not call parse_records on it
        """
        self._parser.parser_max_n_files_for_parsability = 0
        self._stream.get_files.return_value = [_FILE_WITH_UNKNOWN_EXTENSION]

        is_available, reason = self._strategy.check_availability_and_parsability(self._stream, Mock(), Mock())

        assert is_available
        assert not self._parser.parse_records.called
        assert self._stream_reader.open_file.called



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from dataclasses import InitVar, dataclass, field
from typing import Any, Mapping, Optional, Union

from airbyte_cdk.sources.declarative.interpolation.interpolated_nested_mapping import InterpolatedNestedMapping, NestedMapping
from airbyte_cdk.sources.declarative.interpolation.interpolated_string import InterpolatedString
from airbyte_cdk.sources.declarative.types import Config, StreamSlice, StreamState


@dataclass
class InterpolatedNestedRequestInputProvider:
    """
    Helper class that generically performs string interpolation on a provided deeply nested dictionary or string input
    """

    parameters: InitVar[Mapping[str, Any]]
    request_inputs: Optional[NestedMapping] = field(default=None)
    config: Config = field(default_factory=dict)
    _interpolator: Union[InterpolatedString, InterpolatedNestedMapping] = field(init=False, repr=False, default=None)
    _request_inputs: Union[str, Mapping[str, str]] = field(init=False, repr=False, default=None)

    def __post_init__(self, parameters: Mapping[str, Any]):

        self._request_inputs = self.request_inputs or {}
        if isinstance(self.request_inputs, str):
            self._interpolator = InterpolatedString(self.request_inputs, default="", parameters=parameters)
        else:
            self._interpolator = InterpolatedNestedMapping(self._request_inputs, parameters=parameters)

    def eval_request_inputs(
        self, stream_state: StreamState, stream_slice: Optional[StreamSlice] = None, next_page_token: Mapping[str, Any] = None
    ) -> Mapping[str, Any]:
        """
        Returns the request inputs to set on an outgoing HTTP request

        :param stream_state: The stream state
        :param stream_slice: The stream slice
        :param next_page_token: The pagination token
        :return: The request inputs to set on an outgoing HTTP request
        """
        kwargs = {"stream_state": stream_state, "stream_slice": stream_slice, "next_page_token": next_page_token}
        return self._interpolator.eval(self.config, **kwargs)



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from .schema_inferrer import SchemaInferrer
from .traced_exception import AirbyteTracedException
from .is_cloud_environment import is_cloud_environment

__all__ = ["AirbyteTracedException", "SchemaInferrer", "is_cloud_environment"]
